{
  module JavaParser_mit_Typen (parser, Stmt(..), Expr(..), BExpr(..),Type) where
import JavaLexer
}

%name statement
%name returnstatement
%name whilestatement
%name ifthenstatement
%name ifthenelsestatement
%tokentype { Token }
%error { parseError }
  
%token 
       ABSTRACT { ABSTRACT }
       BOOLEAN { BOOLEAN }
       BREAK { BREAK }
       CASE { CASE }
       CATCH { CATCH }
       CHAR  { CHAR  }
       CLASS { CLASS}
       CONTINUE { CONTINUE }
       DEFAULT { DEFAULT }
       DO { DO }
       ELSE { ELSE }
       EXTENDS { EXTENDS }
       FINALLY { FINALLY }
       FOR { FOR }
       IF { IF }
       INSTANCEOF { INSTANCEOF }
       INT { INT }
       NEW { NEW }
       PRIVATE { PRIVATE }
       PROTECTED { PROTECTED }
       PUBLIC { PUBLIC }
       RETURN { RETURN }
       STATIC { STATIC }
       SUPER { SUPER }
       SWITCH { SWITCH }
       THIS { THIS }
       THROW { THROW }
       THROWS { THROWS }
       TRY { TRY }
       VOID { VOID }
       WHILE { WHILE }
       INTLITERAL { INTLITERAL $$}
       BOOLLITERAL { BOOLLITERAL $$}
       JNULL { JNULL }
       CHARLITERAL { CHARLITERAL $$}
       STRINGLITERAL { STRINGLITERAL $$}
       IDENTIFIER { IDENTIFIER $$}
       EQUAL { EQUAL }
       LESSEQUAL { LESSEQUAL }
       GREATEREQUAL { GREATEREQUAL }
       NOTEQUAL { NOTEQUAL }
       LOGICALOR { LOGICALOR }
       LOGICALAND { LOGICALAND }
       INCREMENT { INCREMENT }
       DECREMENT { DECREMENT }
       SHIFTLEFT { SHIFTLEFT }
       SHIFTRIGHT { SHIFTRIGHT }
       UNSIGNEDSHIFTRIGHT { UNSIGNEDSHIFTRIGHT }
       SIGNEDSHIFTRIGHT { SIGNEDSHIFTRIGHT }
       PLUSEQUAL { PLUSEQUAL }
       MINUSEQUAL { MINUSEQUAL }
       TIMESEQUAL { TIMESEQUAL }
       DIVIDEEQUAL { DIVIDEEQUAL }
       ANDEQUAL { ANDEQUAL }
       OREQUAL { OREQUAL }
       XOREQUAL { XOREQUAL }
       MODULOEQUAL { MODULOEQUAL }
       SHIFTLEFTEQUAL { SHIFTLEFTEQUAL }
       SIGNEDSHIFTRIGHTEQUAL { SIGNEDSHIFTRIGHTEQUAL }
       UNSIGNEDSHIFTRIGHTEQUAL { UNSIGNEDSHIFTRIGHTEQUAL }
       LBRACE { LBRACE }
       RBRACE { RBRACE }
       LBRACKET { LBRACKET }
       RBRACKET { RBRACKET }
       LSQBRACKET { LSQBRACKET }
       RSQBRACKET { RSQBRACKET }
       SEMICOLON { SEMICOLON }
       DOT { DOT }
       ASSIGN { ASSIGN }
       LESS { LESS }
       GREATER { GREATER }
       EXCLMARK { EXCLMARK }
       TILDE { TILDE }
       QUESMARK { QUESMARK }
       COLON { COLON }
       PLUS { PLUS }
       MINUS { MINUS }
       MUL { MUL }
       DIV { DIV }
       MOD { MOD }
       AND { AND }
       OR { OR }
       XOR { XOR }
       SHARP { SHARP }
       ARROW { ARROW }

%%



statement        : ifthenstatement { $1 }
		 | ifthenelsestatement { $1 }
		 | whilestatement { $1 }
                 | block { $1 }
		 | emptystatement { $1 }
		 | returnstatement { $1 }

ifthenstatement  : IF LBRACE bexpression  RBRACE  statement { If($3, $5, Nothing) }

ifthenelsestatement : IF LBRACE bexpression  RBRACE statement ELSE statement  {If($3, $5, Just $7)  }

whilestatement   : WHILE LBRACE bexpression  RBRACE  statement { While($3, $5) }

block            : LBRACKET   RBRACKET { Block([Empty]) }
                 | LBRACKET  statements  RBRACKET { Block($2) }

statements  : statement { [$1] }
            | statements statement { $1 ++ [$2] }

emptystatement	 :  SEMICOLON  { Empty }

returnstatement  : RETURN  SEMICOLON  { Return Nothing }
                 | RETURN expression  SEMICOLON { Return (Just $2) }

bexpression : BOOLLITERAL { T -- eine wenig ungenau, nimmt alle boolliterals}  

expression :  INTLITERAL { One -- ein wenig ungenau, nimmt alle intliterals}

{

type Type = String
data Stmt = If(BExpr, Stmt , Maybe Stmt)
          | While( BExpr , Stmt )
          | Block([Stmt])
          | Empty
          | Return( Maybe Expr )
          | TypedStmt(Stmt, Type)
          deriving (Show)
          
data BExpr = T
           | TypedBExpr(BExpr, Type)
     deriving (Show)

data Expr = One
          | TypedExpr(Expr, Type)
     deriving (Show)
  
parseError :: [Token] -> a
parseError _ = error "Parse error"

--parser :: String -> MiniFunkExpr
parser = statement . alexScanTokens

main = do
  s <- readFile "fst.stmt"
  --s <- getContents
  --print ((expr . alexScanTokens) s)
  print (parser s)
}

